package com.example.wfwebapi.controller;

import com.example.wfwebapi.model.Image;
import com.example.wfwebapi.model.User;
import com.example.wfwebapi.repository.ImageRepository;
import com.example.wfwebapi.repository.UserRepository;
import com.example.wfwebapi.security.JwtTokenProvider;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.http.*;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class AuthControllerTest {

    @LocalServerPort
    private int port;

    @Autowired
    private AuthController authController;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private ImageRepository imageRepository;

    @Autowired
    private JwtTokenProvider tokenProvider;

    @Autowired
    private TestRestTemplate restTemplate;

    private String getUrl(String path) {
        return "http://localhost:" + port + "/api/v1/auth" + path;
    }

    @BeforeEach
    void setup() {
        userRepository.deleteAll();
        imageRepository.deleteAll();

        Image image = new Image();
        image.setImgPath("fake/path.png");
        image = imageRepository.save(image);

        User user = new User();
        user.setUsername("john");
        user.setPassword("password123");
        user.setEmail("john@example.com");
        user.setCreation(java.time.LocalDate.now());
        user.setImage(image);

        userRepository.save(user);
    }

    @Test
    void contextLoads() {
        assertNotNull(authController);
        assertNotNull(userRepository);
        assertNotNull(tokenProvider);
    }

    @Test
    void testLoginSuccess() {
        AuthController.AuthRequest request = new AuthController.AuthRequest();
        request.setUsername("john");
        request.setPassword("password123");

        ResponseEntity<AuthController.AuthResponse> response = restTemplate.postForEntity(
                getUrl("/login"), request, AuthController.AuthResponse.class
        );

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertNotNull(response.getBody());
        assertNotNull(response.getBody().getToken());
    }

    @Test
    void testLoginInvalidPassword() {
        AuthController.AuthRequest request = new AuthController.AuthRequest();
        request.setUsername("john");
        request.setPassword("wrongpass");

        ResponseEntity<String> response = restTemplate.postForEntity(
                getUrl("/login"), request, String.class
        );

        assertEquals(HttpStatus.UNAUTHORIZED, response.getStatusCode());
        assertEquals("Mot de passe invalide", response.getBody());
    }

    @Test
    void testLoginUserNotFound() {
        AuthController.AuthRequest request = new AuthController.AuthRequest();
        request.setUsername("unknown");
        request.setPassword("any");

        ResponseEntity<String> response = restTemplate.postForEntity(
                getUrl("/login"), request, String.class
        );

        assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, response.getStatusCode());
        assertTrue(response.getBody().contains("Utilisateur non trouvé"));
    }
}
