package com.example.wfwebapi.controller;

import com.example.wfwebapi.model.*;
import com.example.wfwebapi.repository.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.http.*;

import java.time.LocalDate;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class FavoriteControllerTest {

    @LocalServerPort
    private int port;

    @Autowired private TestRestTemplate restTemplate;

    @Autowired private FavoriteRepository favoriteRepository;
    @Autowired private QuoteRepository quoteRepository;
    @Autowired private UserRepository userRepository;
    @Autowired private ImageRepository imageRepository;
    @Autowired private CaracterRepository caracterRepository;
    @Autowired private SourceRepository sourceRepository;

    private User user;
    private Quote quote;

    private String getUrl(String path) {
        return "http://localhost:" + port + "/api/v1/favorite" + path;
    }

    @BeforeEach
    void setup() {
        favoriteRepository.deleteAll();
        quoteRepository.deleteAll();
        caracterRepository.deleteAll();
        sourceRepository.deleteAll();
        userRepository.deleteAll();
        imageRepository.deleteAll();

        Image image = new Image();
        image.setImgPath("fav.png");
        imageRepository.save(image);

        user = new User();
        user.setUsername("testuser");
        user.setPassword("secret");
        user.setEmail("test@ex.com");
        user.setCreation(LocalDate.now());
        user.setImage(image);
        userRepository.save(user);

        Caracter caracter = new Caracter();
        caracter.setCaracter("Sérieux");
        caracter.setImage(image);
        caracterRepository.save(caracter);

        Source source = new Source();
        source.setTitle("Source X");
        source.setDateS(1999);
        sourceRepository.save(source);

        quote = new Quote();
        quote.setContent("La connaissance est le pouvoir.");
        quote.setLangue("fr");
        quote.setIsValide(true);
        quote.setLikes(100);
        quote.setReason("Validée");
        quote.setCaracter(caracter);
        quote.setSource(source);
        quote.setUserVerif(user);
        quoteRepository.save(quote);
    }

    @Test
    void testAddAndGetFavorite() {
        ResponseEntity<Void> postResponse = restTemplate.postForEntity(
                getUrl("?user=" + user.getId() + "&quote=" + quote.getId()),
                null,
                Void.class
        );

        assertEquals(HttpStatus.OK, postResponse.getStatusCode());

        ResponseEntity<Favorite[]> getResponse = restTemplate.getForEntity(
                getUrl("/" + user.getId()),
                Favorite[].class
        );

        assertEquals(HttpStatus.OK, getResponse.getStatusCode());
        assertEquals(1, getResponse.getBody().length);
        assertEquals(quote.getId(), getResponse.getBody()[0].getQuote().getId());
    }

    @Test
    void testDeleteFavorite() {
        Favorite favorite = new Favorite();
        favorite.setId(new FavoriteId(user.getId(), quote.getId()));
        favorite.setUser(user);
        favorite.setQuote(quote);
        favoriteRepository.save(favorite);

        ResponseEntity<Void> response = restTemplate.exchange(
                getUrl("?user=" + user.getId() + "&quote=" + quote.getId()),
                HttpMethod.DELETE,
                null,
                Void.class
        );

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertFalse(favoriteRepository.existsById(new FavoriteId(user.getId(), quote.getId())));
    }

    @Test
    void testDeleteFavoriteNotFound() {
        ResponseEntity<String> response = restTemplate.exchange(
                getUrl("?user=9999&quote=9999"),
                HttpMethod.DELETE,
                null,
                String.class
        );

        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Favorite non trouvée"));
    }
}
