package com.example.wfwebapi.controller;

import com.example.wfwebapi.model.Image;
import com.example.wfwebapi.repository.ImageRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.http.*;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class ImageControllerTest {

    @LocalServerPort
    private int port;

    @Autowired private TestRestTemplate restTemplate;
    @Autowired private ImageRepository imageRepository;

    private String getUrl(String path) {
        return "http://localhost:" + port + "/api/v1/image" + path;
    }

    @BeforeEach
    void setup() {
        imageRepository.deleteAll();
    }

    @Test
    void testCreateAndGetImage() {
        Image image = new Image();
        image.setImgPath("assets/img/test.png");

        ResponseEntity<Image> postResponse = restTemplate.postForEntity(
                getUrl(""), image, Image.class);

        assertEquals(HttpStatus.OK, postResponse.getStatusCode());
        Image created = postResponse.getBody();
        assertNotNull(created);
        assertNotNull(created.getId());

        ResponseEntity<Image> getResponse = restTemplate.getForEntity(
                getUrl("/" + created.getId()), Image.class);

        assertEquals(HttpStatus.OK, getResponse.getStatusCode());
        assertEquals("assets/img/test.png", getResponse.getBody().getImgPath());
    }

    @Test
    void testUpdateImage() {
        Image image = new Image();
        image.setImgPath("original.png");
        image = imageRepository.save(image);

        image.setImgPath("updated.png");

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<Image> entity = new HttpEntity<>(image, headers);

        ResponseEntity<Image> putResponse = restTemplate.exchange(
                getUrl(""), HttpMethod.PUT, entity, Image.class);

        assertEquals(HttpStatus.OK, putResponse.getStatusCode());
        assertEquals("updated.png", putResponse.getBody().getImgPath());
    }

    @Test
    void testDeleteImage() {
        Image image = new Image();
        image.setImgPath("todelete.png");
        image = imageRepository.save(image);

        ResponseEntity<Void> deleteResponse = restTemplate.exchange(
                getUrl("?id=" + image.getId()), HttpMethod.DELETE, null, Void.class);

        assertEquals(HttpStatus.OK, deleteResponse.getStatusCode());
        assertFalse(imageRepository.existsById(image.getId()));
    }

    @Test
    void testGetImageNotFound() {
        ResponseEntity<String> response = restTemplate.getForEntity(
                getUrl("/9999"), String.class);

        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Image non trouvée"));
    }

    @Test
    void testDeleteImageNotFound() {
        ResponseEntity<String> response = restTemplate.exchange(
                getUrl("?id=9999"), HttpMethod.DELETE, null, String.class);

        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Image non trouvée"));
    }
}
