package com.example.wfwebapi.controller;

import com.example.wfwebapi.model.Image;
import com.example.wfwebapi.model.Quiz;
import com.example.wfwebapi.repository.ImageRepository;
import com.example.wfwebapi.repository.QuizRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.http.*;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class QuizControllerTest {

    @LocalServerPort
    private int port;

    @Autowired private TestRestTemplate restTemplate;
    @Autowired private QuizRepository quizRepository;
    @Autowired private ImageRepository imageRepository;

    private String getUrl(String path) {
        return "http://localhost:" + port + "/api/v1/quiz" + path;
    }

    private Image image;

    @BeforeEach
    void setup() {
        quizRepository.deleteAll();
        imageRepository.deleteAll();

        image = new Image();
        image.setImgPath("quiz/image.png");
        imageRepository.save(image);
    }

    @Test
    void testCreateAndGetQuiz() {
        Quiz quiz = new Quiz();
        quiz.setTitle("Histoire");
        quiz.setImage(image);
        quiz.setNbQuest(5);

        ResponseEntity<Quiz> postResponse = restTemplate.postForEntity(getUrl(""), quiz, Quiz.class);
        assertEquals(HttpStatus.OK, postResponse.getStatusCode());
        Quiz created = postResponse.getBody();
        assertNotNull(created);
        assertNotNull(created.getId());

        ResponseEntity<Quiz> getResponse = restTemplate.getForEntity(getUrl("/" + created.getId()), Quiz.class);
        assertEquals(HttpStatus.OK, getResponse.getStatusCode());
        assertEquals("Histoire", getResponse.getBody().getTitle());
    }

    @Test
    void testUpdateQuiz() {
        Quiz quiz = new Quiz();
        quiz.setTitle("Sciences");
        quiz.setImage(image);
        quiz.setNbQuest(10);
        quiz = quizRepository.save(quiz);

        quiz.setTitle("Sciences - Maj");
        quiz.setNbQuest(12);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<Quiz> entity = new HttpEntity<>(quiz, headers);

        ResponseEntity<Quiz> putResponse = restTemplate.exchange(getUrl(""), HttpMethod.PUT, entity, Quiz.class);
        assertEquals(HttpStatus.OK, putResponse.getStatusCode());
        assertEquals("Sciences - Maj", putResponse.getBody().getTitle());
        assertEquals(12, putResponse.getBody().getNbQuest());
    }

    @Test
    void testDeleteQuiz() {
        Quiz quiz = new Quiz();
        quiz.setTitle("À supprimer");
        quiz.setImage(image);
        quiz.setNbQuest(3);
        quiz = quizRepository.save(quiz);

        ResponseEntity<Void> deleteResponse = restTemplate.exchange(
                getUrl("?id=" + quiz.getId()), HttpMethod.DELETE, null, Void.class);
        assertEquals(HttpStatus.OK, deleteResponse.getStatusCode());
        assertFalse(quizRepository.existsById(quiz.getId()));
    }

    @Test
    void testGetQuizNotFound() {
        ResponseEntity<String> response = restTemplate.getForEntity(getUrl("/9999"), String.class);
        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Quiz non trouvé"));
    }

    @Test
    void testDeleteQuizNotFound() {
        ResponseEntity<String> response = restTemplate.exchange(
                getUrl("?id=9999"), HttpMethod.DELETE, null, String.class);
        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Quiz non trouvé"));
    }

    @Test
    void testGetAllQuizzes() {
        Quiz q1 = new Quiz();
        q1.setTitle("Test 1");
        q1.setImage(image);
        q1.setNbQuest(5);

        Quiz q2 = new Quiz();
        q2.setTitle("Test 2");
        q2.setImage(image);
        q2.setNbQuest(10);

        quizRepository.save(q1);
        quizRepository.save(q2);

        ResponseEntity<Quiz[]> response = restTemplate.getForEntity(getUrl(""), Quiz[].class);
        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(2, response.getBody().length);
    }
}
