package com.example.wfwebapi.controller;

import com.example.wfwebapi.model.*;
import com.example.wfwebapi.repository.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.test.web.server.LocalServerPort;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.*;

import java.util.LinkedHashMap;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class QuoteControllerTest {

    @LocalServerPort
    private int port;

    @Autowired private TestRestTemplate restTemplate;
    @Autowired private QuoteRepository quoteRepository;
    @Autowired private UserRepository userRepository;
    @Autowired private SourceRepository sourceRepository;
    @Autowired private CaracterRepository caracterRepository;

    private User user;
    private Source source;
    private Caracter caracter;

    private String getUrl(String path) {
        return "http://localhost:" + port + "/api/v1/quote" + path;
    }

    @BeforeEach
    void setup() {
        quoteRepository.deleteAll();
        userRepository.deleteAll();
        sourceRepository.deleteAll();
        caracterRepository.deleteAll();

        Image img = new Image();
        img.setImgPath("img.png");

        caracter = new Caracter();
        caracter.setCaracter("Test Character");
        caracter.setImage(img);
        caracter = caracterRepository.save(caracter);

        source = new Source();
        source.setTitle("Test Source");
        source.setDateS(2020);
        source = sourceRepository.save(source);

        user = new User();
        user.setUsername("verif");
        user.setEmail("v@v.fr");
        user.setPassword("pass");
        user.setImage(img);
        user.setCreation(java.time.LocalDate.now());
        user = userRepository.save(user);
    }

    @Test
    void testCreateAndGetQuote() {
        Quote quote = new Quote();
        quote.setContent("Ceci est une citation.");
        quote.setLangue("fr");
        quote.setLikes(15);
        quote.setIsValide(true);
        quote.setReason("valide");
        quote.setCaracter(caracter);
        quote.setSource(source);
        quote.setUserVerif(user);

        ResponseEntity<EntityModel> postResponse = restTemplate.postForEntity(
                getUrl(""), quote, EntityModel.class);
        assertEquals(HttpStatus.OK, postResponse.getStatusCode());
        assertNotNull(postResponse.getBody());

        Long id = ((Number)((LinkedHashMap)postResponse.getBody().getContent()).get("id")).longValue();
        ResponseEntity<EntityModel> getResponse = restTemplate.getForEntity(
                getUrl("/" + id), EntityModel.class);
        assertEquals(HttpStatus.OK, getResponse.getStatusCode());
    }

    @Test
    void testUpdateQuote() {
        Quote quote = new Quote();
        quote.setContent("Ancienne");
        quote.setLangue("fr");
        quote.setLikes(1);
        quote.setIsValide(false);
        quote.setReason("...");
        quote.setCaracter(caracter);
        quote.setSource(source);
        quote.setUserVerif(user);
        quote = quoteRepository.save(quote);

        quote.setContent("Nouvelle version");
        quote.setLangue("en");
        quote.setLikes(100);
        quote.setIsValide(true);
        quote.setReason("modifiée");

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<Quote> request = new HttpEntity<>(quote, headers);

        ResponseEntity<EntityModel> response = restTemplate.exchange(
                getUrl(""), HttpMethod.PUT, request, EntityModel.class);

        assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    void testDeleteQuote() {
        Quote quote = new Quote();
        quote.setContent("À supprimer");
        quote.setLangue("fr");
        quote.setLikes(0);
        quote.setIsValide(false);
        quote.setReason("test");
        quote.setCaracter(caracter);
        quote.setSource(source);
        quote.setUserVerif(user);
        quote = quoteRepository.save(quote);

        ResponseEntity<Void> response = restTemplate.exchange(
                getUrl("/delete?idQuote=" + quote.getId()), HttpMethod.DELETE, null, Void.class);
        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertFalse(quoteRepository.existsById(quote.getId()));
    }

    @Test
    void testGetQuoteNotFound() {
        ResponseEntity<String> response = restTemplate.getForEntity(
                getUrl("/9999"), String.class);
        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Citation non trouvée"));
    }

    @Test
    void testGetAllQuotes() {
        Quote quote1 = new Quote();
        quote1.setContent("Q1");
        quote1.setLangue("fr");
        quote1.setLikes(5);
        quote1.setIsValide(true);
        quote1.setReason("ok");
        quote1.setCaracter(caracter);
        quote1.setSource(source);
        quote1.setUserVerif(user);

        Quote quote2 = new Quote();
        quote2.setContent("Q2");
        quote2.setLangue("en");
        quote2.setLikes(10);
        quote2.setIsValide(true);
        quote2.setReason("ok");
        quote2.setCaracter(caracter);
        quote2.setSource(source);
        quote2.setUserVerif(user);

        quoteRepository.save(quote1);
        quoteRepository.save(quote2);

        ResponseEntity<String> response = restTemplate.getForEntity(
                getUrl("/all?index=0&count=10"), String.class);
        assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    void testGetDailyQuote() {
        Quote quote = new Quote();
        quote.setContent("Jour");
        quote.setLangue("fr");
        quote.setLikes(3);
        quote.setIsValide(true);
        quote.setReason("OK");
        quote.setCaracter(caracter);
        quote.setSource(source);
        quote.setUserVerif(user);
        quoteRepository.save(quote);

        ResponseEntity<EntityModel> response = restTemplate.getForEntity(
                getUrl("/dailyquote?year=2023&month=12&day=1&lang=fr"), EntityModel.class);
        assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    void testGetDailyQuoteNotFound() {
        ResponseEntity<String> response = restTemplate.getForEntity(
                getUrl("/dailyquote?year=2023&month=12&day=1&lang=zz"), String.class);
        assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
        assertTrue(response.getBody().contains("Aucune citation trouvée"));
    }
}
